using FitConnect.Models;
using FitConnect.Services.Interfaces;
using FitConnect.Services.Models.v1.Routes;
using Microsoft.Extensions.Logging;

namespace FitConnect.Services;

internal class RouteService : RestCallService, IRouteService {
    public RouteService(string baseUrl, string version = "v1",
        ILogger? logger = null) : base($"{baseUrl}/{version}", logger) {
    }

    /// <summary>
    ///     Returns the destination id for the given intent.
    /// </summary>
    /// <param name="leikaKey"></param>
    /// <param name="ags"></param>
    /// <param name="ars"></param>
    /// <param name="areaId"></param>
    /// <returns></returns>
    public async Task<List<Route>> FindDestinationIdAsync(string leikaKey,
        string? ags, string? ars,
        string? areaId, int offset = 0, int limit = 500) {
        ValidateParametersForGetDestinationId(ags, ars, areaId);

        var result = await RestCall<RoutesListDto>($"/routes?leikaKey={leikaKey}" +
                                                   (ags != null ? $"&ags={ags}" : "") +
                                                   (ars != null ? $"&ars={ars}" : "") +
                                                   (areaId != null ? $"&areaId={areaId}" : "") +
                                                   $"&offset={offset}&limit={limit}",
            HttpMethod.Get);

        return result?.Routes?.ToList() ?? new List<Route>();
    }

    public async Task<AreaList?> FindAreas(List<string> filter, int offset = 0, int limit = 100) {
        var areSearchExpression = filter.Aggregate((a, b) => $"{a}&areaSearchexpression={b}");
        var result = await RestCall<AreaList>(
            $"/areas?areaSearchexpression={areSearchExpression}&offset={offset}&limit={limit}",
            HttpMethod.Get);

        return result;
    }

    private void ValidateParametersForGetDestinationId(string? ags, string? ars, string? areaId) {
        if (ars == null && ags == null && areaId == null)
            throw new ArgumentException("Either ars, ags or areaId must be specified.");

        var paramCount = 0;
        if (ags != null) paramCount++;
        if (ars != null) paramCount++;
        if (areaId != null) paramCount++;

        if (paramCount != 1)
            throw new ArgumentException("Only one of ars, ags or areaId must be specified.");
    }
}
